@@ -5,6 +5,7 @@ from typing import List, Tuple
 import apsw
 import argon2
 
+from .conf import config
 from .database.common import run_sql, run_sql_many
 
 
@@ -16,23 +17,29 @@ def save_title(title):
         name,
         site,
         cover_ext,
+        is_webtoon,
         chapters,
         alt_names,
-        descriptions
+        descriptions,
+        descriptions_format
     ) VALUES (
         :id,
         :name,
         :site,
         :cover_ext,
+        :is_webtoon,
         :chapters,
         :alt_names,
-        :descriptions
+        :descriptions,
+        :descriptions_format
     ) ON CONFLICT (id, site) DO UPDATE SET
         name=excluded.name,
         cover_ext=excluded.cover_ext,
+        is_webtoon=excluded.is_webtoon,
         chapters=excluded.chapters,
         alt_names=excluded.alt_names,
         descriptions=excluded.descriptions,
+        descriptions_format=excluded.descriptions_format,
         updated_at=datetime('now')
     ;
     """,
@@ -41,9 +48,11 @@ def save_title(title):
             "name": title["name"],
             "site": title["site"],
             "cover_ext": title["cover_ext"],
+            "is_webtoon": title["is_webtoon"],
             "chapters": json.dumps(title["chapters"]),
             "alt_names": json.dumps(title["alt_names"]),
             "descriptions": json.dumps(title["descriptions"]),
+            "descriptions_format": title["descriptions_format"],
         },
     )
 
@@ -51,7 +60,8 @@ def save_title(title):
 def load_title(site, title_id, user_id=None):
     result = run_sql(
         """
-        SELECT id, name, site, cover_ext, chapters, alt_names, descriptions
+        SELECT id, name, site, cover_ext, is_webtoon, chapters, alt_names, descriptions,
+               descriptions_format
         FROM title
         WHERE id = ?
           AND site = ?;
@@ -106,8 +116,7 @@ def save_chapter(chapter):
         name,
         pages,
         pages_alt,
-        groups,
-        is_webtoon
+        groups
     ) VALUES (
         :id,
         :title_id,
@@ -117,8 +126,7 @@ def save_chapter(chapter):
         :name,
         :pages,
         :pages_alt,
-        :groups,
-        :is_webtoon
+        :groups
     ) ON CONFLICT (id, title_id, site) DO UPDATE SET
         num_major=excluded.num_major,
         num_minor=excluded.num_minor,
@@ -126,7 +134,6 @@ def save_chapter(chapter):
         pages=excluded.pages,
         pages_alt=excluded.pages_alt,
         groups=excluded.groups,
-        is_webtoon=excluded.is_webtoon,
         updated_at=datetime('now')
     ;
     """,
@@ -140,7 +147,6 @@ def save_chapter(chapter):
             "pages": json.dumps(chapter["pages"]),
             "pages_alt": json.dumps(chapter["pages_alt"]),
             "groups": json.dumps(chapter["groups"]),
-            "is_webtoon": chapter["is_webtoon"],
         },
     )
 
@@ -149,7 +155,7 @@ def load_chapter(site, title_id, chapter_id, ignore_old=True):
     updated_at = "datetime('now', '-1 days')" if ignore_old else "'1980-01-01'"
     result = run_sql(
         f"""
-        SELECT id, title_id, site, num_major, num_minor, name, pages, pages_alt, groups, is_webtoon
+        SELECT id, title_id, site, num_major, num_minor, name, pages, pages_alt, groups
         FROM chapter
         WHERE site=? AND title_id=? AND id=? AND updated_at > {updated_at};
         """,
@@ -267,7 +273,16 @@ def get_followed_titles(user_id):
 
         t["chapters"] = chapters
 
-    return sorted(titles, key=lambda t: len(t["chapters"]), reverse=True)
+    def sort_key_func(title):
+        """
+        Prioritize titles that:
+        - Have unread chapters
+        - Are from mangasee (because mangadex is currently broken)
+        """
+        has_chapters = 1 if len(title["chapters"]) > 0 else 0
+        return (has_chapters, title["site"])
+
+    return sorted(titles, key=sort_key_func, reverse=True)
 
 
 def read(user_id, site, title_id, chapter_id):
@@ -288,9 +303,13 @@ def unread(user_id, site, title_id, chapter_id):
     )
 
 
-def find_outdated_titles(since="-6 hours"):
+def find_outdated_titles(since=f"-{config.MANGA_HOURS_UNTIL_OUTDATED} hours"):
     return run_sql(
-        "SELECT id, site FROM title WHERE updated_at <= datetime('now', ?);", (since,)
+        """
+        SELECT id, name, site FROM title WHERE updated_at <= datetime('now', ?)
+        ORDER BY updated_at;
+        """,
+        (since,),
     )
 
 
@@ -390,3 +409,21 @@ def delete_expired_tokens():
         return_num_affected=True,
     )
     return num_deleted
+
+
+def is_manga_page_url(url):
+    """
+    Checks if url exists in db as a page image.
+    This is currently used to avoid abuse of our /proxy/ endpoint.
+    """
+    is_page = run_sql(
+        "SELECT 1 FROM chapter, json_each(pages) WHERE value = ? LIMIT 1;", (url,)
+    ) == [1]
+    if is_page:
+        return True
+
+    is_page_alt = run_sql(
+        "SELECT 1 FROM chapter, json_each(pages_alt) WHERE value = ? LIMIT 1;", (url,)
+    ) == [1]
+
+    return is_page_alt
